package com.jz.dmp.modules.service.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.jz.agent.service.DmpDsAgentService;
import com.jz.common.constant.JsonResult;
import com.jz.common.constant.ResultCode;
import com.jz.common.page.PageInfoResponse;
import com.jz.common.persistence.BaseService;
import com.jz.common.utils.AzkabanApiUtils2;
import com.jz.common.utils.FileUtils;
import com.jz.common.utils.JsonMapper;
import com.jz.common.utils.ZipUtils;
import com.jz.dmp.agent.DmpAgentResult;
import com.jz.dmp.modules.controller.DataIntegration.bean.SourceDbNameListDto;
import com.jz.dmp.modules.controller.DataIntegration.bean.TaskListPageDto;
import com.jz.dmp.modules.controller.DataIntegration.bean.TaskListPageReq;
import com.jz.dmp.modules.dao.DmpProjectDao;
import com.jz.dmp.modules.dao.OfflineSynchDao;
import com.jz.dmp.modules.model.DmpAgentDatasourceInfo;
import com.jz.dmp.modules.model.DmpProjectSystemInfo;
import com.jz.dmp.modules.model.DmpSyncingDatasource;
import com.jz.dmp.modules.service.OfflineSynchService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description:离线同步服务层
 * @ClassName: OfflineSynchServiceImpl
 * @Author Bellamy
 * @Date 2020/12/20
 */

@Service("offlineSynchService")
@Transactional
public class OfflineSynchServiceImpl implements OfflineSynchService {

    @Value("${spring.public-key}")
    private String publicKey;

    @Autowired
    private OfflineSynchDao offlineSynchDao;

    @Autowired
    private DmpProjectDao dmpProjectDao;

    @Autowired
    private DmpDsAgentService dmpDsAgentServiceImp;

    @Override
    public PageInfoResponse<TaskListPageDto> queryTaskListPage(TaskListPageReq taskListPageReq) throws Exception {
        PageInfoResponse<TaskListPageDto> pageInfoResponse = new PageInfoResponse<>();

        Map param = new HashMap();
        //param.put("userId", user.getUserId());
        //param.put("pageNum", pageBean.getPageNum());
        //param.put("pageSize", pageBean.getPageSize());
        PageHelper.startPage(taskListPageReq.getPageNum(), taskListPageReq.getPageSize());
        List<TaskListPageDto> list = offlineSynchDao.queryTaskListPage(taskListPageReq);
        PageInfo<TaskListPageDto> pageInfo = new PageInfo<>(list);

        pageInfoResponse.setCode(ResultCode.SUCCESS);
        pageInfoResponse.setMessage("查询成功");
        pageInfoResponse.setData(pageInfo);
        return pageInfoResponse;
    }

    @Override
    public JsonResult<List<SourceDbNameListDto>> querygSourceDbList(Integer projectId) throws Exception {
        Map map = new HashMap();
        map.put("projectId", projectId); //项目id
        map.put("isEnableSource", "1");
        List<SourceDbNameListDto> list = offlineSynchDao.querygSourceDbList(map);
        return new JsonResult(ResultCode.SUCCESS, list);
    }

    @Override
    public JsonResult querygSourceTableList(Integer sourceDbId, String targetName) throws Exception {
        //通过源数据库id ，查询数据源配置
        DmpAgentDatasourceInfo dsInfo = offlineSynchDao.querySourceDbInfoBySourceId(sourceDbId);
        //解码源数据库密码
        if (StringUtils.isNotBlank(dsInfo.getPassword())) {
            dsInfo.setPassword(new BaseService().decode(dsInfo.getPassword(), publicKey));
        }
        if (StringUtils.isNotBlank(targetName)) {
            dsInfo.setTargetFileName(targetName);
        }
        //创建jdbc,获取源数据表
        DmpAgentResult rst = dmpDsAgentServiceImp.getTableNameList(dsInfo);
        if (!rst.getCode().val().equals("200")) {
            return new JsonResult(rst.getCode(), rst.getMessage());
        } else {
            //成功
            rst.setResult(JsonMapper.fromJsonString(rst.getMessage(), List.class));
            return new JsonResult(ResultCode.SUCCESS, rst);
        }
    }

    /**
     * 立即运行
     */
    @Override
    public JsonResult taskRunNowByTaskId(String taskId) throws Exception {
        //根据任务id，查询DMP资源导航树和DW系统配置信息
        Map<String,Object> map = offlineSynchDao.selectNavigationTreeByTaskId(taskId);
        if (map.size() > 0 && map != null) {
            this.publish(map);
        }
        return null;
    }

    /**
     * 发布流程
     */
    private boolean publish(Map<String,Object> map) {
        String taskId = map.get("taskId").toString(); //任务id
        String projectId = map.get("projectId").toString(); //项目id
        String treeName = map.get("treeName").toString(); //流程名称
        String azkabanExectorXmlExec = map.get("azkabanExectorXmlExec").toString(); //执行数据同步任务命令
        String azkabanLocalTaskFilePath = map.get("azkabanLocalTaskFilePath").toString(); //文件路径
        String azkabanMonitorUrl = map.get("azkabanMonitorUrl").toString();//AZKABAN WEB服务地址

        String azkabanJobCommand = "command=" + azkabanExectorXmlExec + " " + projectId + " ${azkaban.flow.flowid} ${azkaban.job.id} ${azkaban.flow.execid} " + treeName;

        /**
         * 当前任务生成文件存放根路径
         */
        String localTaskPath = azkabanLocalTaskFilePath + "/" + projectId + "/local/" + taskId;
        File localTaskFile = new File(localTaskPath);
        if (!localTaskFile.exists()) {
            localTaskFile.mkdirs();
        }

        String localTaskSourcePath = localTaskPath + "/source/";
        File localTaskSourceFile = new File(localTaskSourcePath);
        if (!localTaskSourceFile.exists()) {
            localTaskSourceFile.mkdirs();
        }

        String localTaskExecArgsPath = localTaskPath + "/execArgs/";
        File localTaskExecArgsFile = new File(localTaskExecArgsPath);
        if (!localTaskExecArgsFile.exists()) {
            localTaskExecArgsFile.mkdirs();
        }

        String localTaskZipPath = localTaskPath + "/target/";
        File localTaskZipFile = new File(localTaskZipPath);
        if (!localTaskZipFile.exists()) {
            localTaskZipFile.mkdirs();
        }

        //先删除上次创建的文件 ---每次都会新建
        FileUtils.deleteDirectory(localTaskPath);


        List<String> contents = new ArrayList<>();

        // 子流程类型
        contents.add("type=command");
        contents.add(azkabanJobCommand);

        // 生成job文件
        String jobFileAbsolutePath = localTaskSourcePath + treeName + ".job";
        FileUtils.write(jobFileAbsolutePath, contents);

        String localZipTargetFileName = treeName + ".zip";
        ZipUtils.zip(localTaskSourcePath, localTaskZipPath, localZipTargetFileName);

        //上传到azkaban todo
        //上次zip包到azkaban
        String localTaskZipAbsolutePath = localTaskZipPath + "/" + localZipTargetFileName;
        AzkabanApiUtils2 azkabanApiUtils = new AzkabanApiUtils2(azkabanMonitorUrl);
        return azkabanApiUtils.loginCreateProjectuploadZipAndExecute("jz_localflow_" + projectId, "local_sync_project", localTaskZipAbsolutePath, treeName);
    }

}
